package org.drools.reteoo;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import org.drools.reteoo.model.Pet;
import org.drools.reteoo.model.Pet.PetType;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.kie.api.KieServices;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.rule.FactHandle;
import org.kie.api.runtime.rule.LiveQuery;
import org.kie.api.runtime.rule.Row;
import org.kie.api.runtime.rule.ViewChangedEventListener;

/**
 * This class tests live queries - different behaviour with RETE and PHREAK.
 */
public class LiveQueriesTest {

    private ArrayList<Object> inserted, updated, deleted;

    private Pet youngCat, oldCat, youngDog, oldDog, youngParrot, oldParrot;

    private FactHandle yC, oC, yD, oD, yP, oP;

    private KieSession kieSession;

    private ViewChangedEventListener listener;

    private LiveQuery query;
   
    @Before
    public void initialize() {

        KieServices ks = KieServices.Factory.get();
        kieSession = ks.getKieClasspathContainer().newKieSession("liveQueriesKSession");

        inserted = new ArrayList<Object>();
        updated = new ArrayList<Object>();
        deleted = new ArrayList<Object>();

        listener = new ViewChangedEventListener() {

            @Override
            public void rowUpdated(Row row) {
                updated.add(row.get("$age"));
            }

            @Override
            public void rowInserted(Row row) {
                inserted.add(row.get("$age"));
            }

            @Override
            public void rowDeleted(Row row) {
                deleted.add(row.get("$age"));
            }
        };

        youngCat = new Pet(PetType.cat, 1);
        oldCat = new Pet(PetType.cat, 10);
        youngDog = new Pet(PetType.dog, 3);
        oldDog = new Pet(PetType.dog, 13);
        youngParrot = new Pet(PetType.parrot, 2);
        oldParrot = new Pet(PetType.parrot, 8);

        yC = kieSession.insert(youngCat);
        oC = kieSession.insert(oldCat);
        yD = kieSession.insert(youngDog);
        oD = kieSession.insert(oldDog);
        yP = kieSession.insert(youngParrot);
        oP = kieSession.insert(oldParrot);

        query = kieSession.openLiveQuery("pets", new Object[] { PetType.cat },
                listener);
    }

    @After
    public void release() {
        kieSession.dispose();
    }
    
    @Test
    public void testSimpleLiveQuery() {

        // there are two cats
        assertEquals(2, inserted.size());
        assertEquals(0, updated.size());
        assertEquals(0, deleted.size());

        assertEquals(1, inserted.get(0));
        assertEquals(10, inserted.get(1));

        // young cat has a birthday
        youngCat.setAge(youngCat.getAge() + 1);
        kieSession.update(yC, youngCat);
        kieSession.fireAllRules();

        assertEquals(2, inserted.size());
        assertEquals(1, updated.size());
        assertEquals(0, deleted.size());

        assertEquals(1, inserted.get(0));
        assertEquals(2, updated.get(0));

        // old cat died
        kieSession.delete(oC);
        kieSession.fireAllRules();

        assertEquals(2, inserted.size());
        assertEquals(1, updated.size());
        assertEquals(1, deleted.size());

        assertEquals(10, deleted.get(0));

        // delete query
        query.close();

        assertEquals(2, inserted.size());
        assertEquals(2, deleted.size());
        assertEquals(1, updated.size());

        assertEquals(10, deleted.get(0));
        assertEquals(2, deleted.get(1));

        // update after delete query
        youngCat.setAge(5);
        kieSession.update(yC, youngCat);
        kieSession.fireAllRules();

        assertEquals(2, inserted.size());
        assertEquals(2, deleted.size());
        assertEquals(1, updated.size());
    }
}
