package org.drools.reteoo;

import java.util.ArrayList;
import static org.junit.Assert.assertEquals;

import org.drools.reteoo.model.Person;

import org.junit.After;
import static org.junit.Assert.assertTrue;
import org.junit.Before;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.definition.type.FactType;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.rule.QueryResults;
import org.kie.api.runtime.rule.QueryResultsRow;
import org.kie.api.runtime.rule.Variable;

/**
 * Test to verify correct functionality of queries using backward chaining
 * with RETEOO vs. PHREAK.
 */
public class QueryBCTest {

    private static final long LOOPS = 100;

    private KieSession ksession;

    private Person petr, pavel, tomas, michaela, magda, monika;

    @Before
    public void prepareKieSession() throws Exception {
        KieServices ks = KieServices.Factory.get();
        KieBase kbase = ks.getKieClasspathContainer().getKieBase("queriesKBase");
        ksession = kbase.newKieSession();

        this.insertFacts();
    }

    @After
    public void disposeSession() {
        if (this.ksession != null) {
            this.ksession.dispose();
        }
    }

    private void insertFacts() {

        petr = new Person("Petr", "canoe", 25);
        pavel = new Person("Pavel", "basketball", 27);
        tomas = new Person("Tomas", "kendo", 23);
        michaela = new Person("Michaela", "films", 20);
        magda = new Person("Magda", "dogs", 17);
        monika = new Person("Monika", "voleyball", 30);

        ksession.insert(petr);
        ksession.insert(pavel);
        ksession.insert(tomas);
        ksession.insert(michaela);
        ksession.insert(magda);
        ksession.insert(monika);
    }

    @Test
    public void testBackwardChaining() throws InstantiationException, IllegalAccessException {

        FactType locationType = ksession.getKieBase().getFactType("org.drools.reteoo.queries",
                                                                  "Location");

        Object table = locationType.newInstance();
        locationType.set(table, "thing", "table");
        locationType.set(table, "location", "office");

        Object cup = locationType.newInstance();
        locationType.set(cup, "thing", "cup");
        locationType.set(cup, "location", "table");

        Object ball = locationType.newInstance();
        locationType.set(ball, "thing", "ball");
        locationType.set(ball, "location", "outside");

        ksession.insert(ball);
        ksession.insert(cup);
        ksession.insert(table);

        QueryResults results = ksession.getQueryResults("isContainedIn", new Object[] { Variable.v, "office" });

        assertEquals(results.size(), 2);

        ArrayList<Object> list = new ArrayList<Object>();

        for (QueryResultsRow qrr : results) {
            list.add(qrr.get("x"));
        }

        assertTrue(list.contains(locationType.get(table, "thing")));
        assertTrue(list.contains(locationType.get(cup, "thing")));
    }

    // Backward chaining test with position annotation
    @Test
    public void testBackwardChainingWithPositionAnnotation() throws InstantiationException, IllegalAccessException {

        FactType locationType = ksession.getKieBase().getFactType("org.drools.reteoo.queries",
                                                                  "Location2");

        Object table = locationType.newInstance();
        locationType.set(table, "location", "office");
        locationType.set(table, "thing", "table");

        Object cup = locationType.newInstance();
        locationType.set(cup, "location", "table");
        locationType.set(cup, "thing", "cup");

        Object ball = locationType.newInstance();
        locationType.set(ball, "location", "outside");
        locationType.set(ball, "thing", "ball");

        ksession.insert(ball);
        ksession.insert(cup);
        ksession.insert(table);

        QueryResults results = ksession.getQueryResults("isContainedIn2", new Object[] { Variable.v, "office" });

        assertEquals(results.size(), 2);

        ArrayList<Object> list = new ArrayList<Object>();

        for (QueryResultsRow qrr : results) {
            list.add(qrr.get("x"));
        }

        assertTrue(list.contains(locationType.get(table, "thing")));
        assertTrue(list.contains(locationType.get(cup, "thing")));
    }

    // Test calling query from other query
    @Test
    public void testCallQueryFromOtherQuery() {

        QueryResults results = ksession.getQueryResults("getPersonsBetween", 19, 26);

        assertEquals(results.size(), 3);

        ArrayList<Person> personList = new ArrayList<Person>();

        for (QueryResultsRow qrr : results) {
            personList.add((Person) qrr.get("p"));
        }

        assertTrue(personList.contains(petr));
        assertTrue(personList.contains(tomas));
        assertTrue(personList.contains(michaela));
    }

}
