package org.drools.reteoo;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import org.drools.core.time.SessionPseudoClock;
import org.drools.reteoo.model.TestEvent;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.rule.QueryResults;
import org.kie.api.runtime.rule.EntryPoint;

/**
 * Test to verify correct functionality of Drools Fusion sliding windows 
 * with RETEOO vs. PHREAK.
 */
public class WindowTest {

    private static final long LOOPS = 100;

    private KieSession ksession;
    
    private List<Long> result;

    @Before
    public void prepareKieSession() throws Exception {
        KieServices ks = KieServices.Factory.get();
        KieBase kbase = ks.getKieClasspathContainer().getKieBase("windowKBase");
        ksession = kbase.newKieSession();

        result = new ArrayList<Long>();
        ksession.setGlobal("result", result);
    }

    @After
    public void disposeSession() {
        if (this.ksession != null) {
            this.ksession.dispose();
        }
        result = null;
    }

    @Test
    public void testTimeWindow() throws InterruptedException {
        EntryPoint entryPoint = ksession.getEntryPoint("EventStream");
        final long results[] = new long[] { 1, 2, 3, 3, 3 };
        TestEvent event;

        for (int i = 0; i < 5; i++) {
            event = new TestEvent(null, "time", null);
            entryPoint.insert(event);
            ksession.fireAllRules();

            assertEquals(results[i], getQueryResult(ksession, "TestTimeWindow", "$eventCount"));
            advanceTime(100);
        }
    }

    @Test
    public void testLengthWindow() {
        EntryPoint entryPoint = ksession.getEntryPoint("EventStream");
        TestEvent event;

        for (int i = 1; i <= LOOPS; i++) {
            event = new TestEvent(null, "length", null);
            entryPoint.insert(event);
            ksession.fireAllRules();

            assertEquals((i < 10 ? i : 10), getQueryResult(ksession, "TestLengthWindow", "$eventCount"));
        }
    }

    private long getQueryResult(KieSession ksession, String query, String result) {
        QueryResults results = ksession.getQueryResults(query);

        assertEquals(results.size(), 1);
        return (Long) results.iterator().next().get(result);
    }
    
    private void advanceTime(final long millis) throws InterruptedException {
        if (ksession.getSessionClock() instanceof SessionPseudoClock) {
            SessionPseudoClock clock = ksession.getSessionClock();
            clock.advanceTime(millis, TimeUnit.MILLISECONDS);
        } else {
            final long timeToSleep = millis;
            if (timeToSleep > 60000) {
                throw new IllegalArgumentException("It is not desired to wait for more than a minute in test.");
            }
            Thread.sleep(timeToSleep);
        }        
    }
}
